/* BdfServer - Copyright (c) 2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.instruction;

import fr.exemole.bdfserver.api.balayage.Balayage;
import fr.exemole.bdfserver.api.configuration.PathConfiguration;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.roles.Role;
import fr.exemole.bdfserver.tools.configuration.PathConfigurationBuilder;
import net.fichotheque.addenda.Addenda;
import net.fichotheque.addenda.Document;
import net.fichotheque.album.Album;
import net.fichotheque.album.Illustration;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.exportation.access.AccessDef;
import net.fichotheque.exportation.balayage.BalayageDescription;
import net.fichotheque.exportation.scrutari.ScrutariExportDef;
import net.fichotheque.exportation.sql.SqlExportDef;
import net.fichotheque.exportation.table.TableExportDescription;
import net.fichotheque.exportation.transformation.TemplateContentDescription;
import net.fichotheque.exportation.transformation.TemplateDescription;
import net.fichotheque.exportation.transformation.TransformationDescription;
import net.fichotheque.selection.SelectionDef;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.sphere.Sphere;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public final class BdfOutputUtils {

    private BdfOutputUtils() {

    }

    public static Corpus getCorpus(OutputParameters parameters) throws ErrorMessageException {
        Corpus corpus = (Corpus) parameters.getResultObject(BdfInstructionConstants.CORPUS_OBJ);
        if (corpus == null) {
            FicheMeta ficheMeta = (FicheMeta) parameters.getResultObject(BdfInstructionConstants.FICHEMETA_OBJ);
            if (ficheMeta != null) {
                corpus = ficheMeta.getCorpus();
            }
        }
        if (corpus == null) {
            corpus = BdfInstructionUtils.getMandatoryCorpus(parameters.getFichotheque(), parameters.getRequestMap());
        }
        parameters.checkSubsetAccess(corpus);
        return corpus;
    }

    public static Thesaurus getThesaurus(OutputParameters parameters) throws ErrorMessageException {
        Thesaurus thesaurus = (Thesaurus) parameters.getResultObject(BdfInstructionConstants.THESAURUS_OBJ);
        if (thesaurus == null) {
            Motcle motcle = (Motcle) parameters.getResultObject(BdfInstructionConstants.MOTCLE_OBJ);
            if (motcle != null) {
                thesaurus = motcle.getThesaurus();
            }
        }
        if (thesaurus == null) {
            thesaurus = BdfInstructionUtils.getMandatoryThesaurus(parameters.getFichotheque(), parameters.getRequestMap());
        }
        parameters.checkSubsetAccess(thesaurus);
        return thesaurus;
    }

    public static Addenda getAddenda(OutputParameters parameters) throws ErrorMessageException {
        Addenda addenda = (Addenda) parameters.getResultObject(BdfInstructionConstants.ADDENDA_OBJ);
        if (addenda == null) {
            Document document = (Document) parameters.getResultObject(BdfInstructionConstants.DOCUMENT_OBJ);
            if (document != null) {
                addenda = document.getAddenda();
            }
        }
        if (addenda == null) {
            addenda = BdfInstructionUtils.getMandatoryAddenda(parameters.getFichotheque(), parameters.getRequestMap());
        }
        parameters.checkSubsetAccess(addenda);
        return addenda;
    }

    public static Album getAlbum(OutputParameters parameters) throws ErrorMessageException {
        Album album = (Album) parameters.getResultObject(BdfInstructionConstants.ALBUM_OBJ);
        if (album == null) {
            Illustration illustration = (Illustration) parameters.getResultObject(BdfInstructionConstants.ILLUSTRATION_OBJ);
            if (illustration != null) {
                album = illustration.getAlbum();
            }
        }
        if (album == null) {
            album = BdfInstructionUtils.getMandatoryAlbum(parameters.getFichotheque(), parameters.getRequestMap());
        }
        parameters.checkSubsetAccess(album);
        return album;
    }

    public static Sphere getSphere(OutputParameters parameters) throws ErrorMessageException {
        Sphere sphere = (Sphere) parameters.getResultObject(BdfInstructionConstants.SPHERE_OBJ);
        if (sphere == null) {
            Redacteur redacteur = (Redacteur) parameters.getResultObject(BdfInstructionConstants.REDACTEUR_OBJ);
            if (redacteur != null) {
                sphere = redacteur.getSphere();
            }
        }
        if (sphere == null) {
            sphere = BdfInstructionUtils.getMandatorySphere(parameters.getFichotheque(), parameters.getRequestMap());
        }
        parameters.checkSubsetAccess(sphere);
        return sphere;
    }

    public static FicheMeta getFicheMeta(OutputParameters parameters) throws ErrorMessageException {
        FicheMeta ficheMeta = (FicheMeta) parameters.getResultObject(BdfInstructionConstants.FICHEMETA_OBJ);
        if (ficheMeta == null) {
            ficheMeta = BdfInstructionUtils.getMandatoryFicheMeta(parameters.getFichotheque(), parameters.getRequestMap(), parameters.getBdfUser());
        }
        return ficheMeta;
    }

    public static Illustration getIllustration(OutputParameters parameters) throws ErrorMessageException {
        Illustration illustration = (Illustration) parameters.getResultObject(BdfInstructionConstants.ILLUSTRATION_OBJ);
        if (illustration == null) {
            illustration = BdfInstructionUtils.getMandatoryIllustration(parameters.getFichotheque(), parameters.getRequestMap());
        }
        return illustration;
    }

    public static Document getDocument(OutputParameters parameters) throws ErrorMessageException {
        Document document = (Document) parameters.getResultObject(BdfInstructionConstants.DOCUMENT_OBJ);
        if (document == null) {
            document = BdfInstructionUtils.getMandatoryDocument(parameters.getFichotheque(), parameters.getRequestMap());
        }
        return document;
    }

    public static Redacteur getRedacteur(OutputParameters parameters) throws ErrorMessageException {
        Redacteur redacteur = (Redacteur) parameters.getResultObject(BdfInstructionConstants.REDACTEUR_OBJ);
        if (redacteur == null) {
            redacteur = BdfInstructionUtils.getMandatoryRedacteur(parameters.getFichotheque(), parameters.getRequestMap());
        }
        return redacteur;
    }

    public static Motcle getMotcle(OutputParameters parameters) throws ErrorMessageException {
        Motcle motcle = (Motcle) parameters.getResultObject(BdfInstructionConstants.MOTCLE_OBJ);
        if (motcle == null) {
            motcle = BdfInstructionUtils.getMandatoryMotcle(parameters.getFichotheque(), parameters.getRequestMap());
        }
        return motcle;
    }

    public static Role getRole(OutputParameters parameters) throws ErrorMessageException {
        Role role = (Role) parameters.getResultObject(BdfInstructionConstants.ROLE_OBJ);
        if (role == null) {
            role = BdfInstructionUtils.getMandatoryRole(parameters.getBdfServer().getPermissionManager(), parameters.getRequestMap());
        }
        return role;
    }

    public static SelectionDef getSelectionDef(OutputParameters parameters) throws ErrorMessageException {
        SelectionDef selectionDefDef = (SelectionDef) parameters.getResultObject(BdfInstructionConstants.SELECTIONDEF_OBJ);
        if (selectionDefDef == null) {
            selectionDefDef = BdfInstructionUtils.getMandatorySelectionDef(parameters.getBdfServer().getSelectionManager(), parameters.getRequestMap());
        }
        return selectionDefDef;
    }

    public static ScrutariExportDef getScrutariExportDef(OutputParameters parameters) throws ErrorMessageException {
        ScrutariExportDef scrutariExportDef = (ScrutariExportDef) parameters.getResultObject(BdfInstructionConstants.SCRUTARIEXPORTDEF_OBJ);
        if (scrutariExportDef == null) {
            scrutariExportDef = BdfInstructionUtils.getMandatoryScrutariExportDef(parameters.getBdfServer().getScrutariExportManager(), parameters.getRequestMap());
        }
        return scrutariExportDef;
    }

    public static SqlExportDef getSqlExportDef(OutputParameters parameters) throws ErrorMessageException {
        SqlExportDef sqlExportDef = (SqlExportDef) parameters.getResultObject(BdfInstructionConstants.SQLEXPORTDEF_OBJ);
        if (sqlExportDef == null) {
            sqlExportDef = BdfInstructionUtils.getMandatorySqlExportDef(parameters.getBdfServer().getSqlExportManager(), parameters.getRequestMap());
        }
        return sqlExportDef;
    }

    public static AccessDef getAccessDef(OutputParameters parameters) throws ErrorMessageException {
        AccessDef accessDef = (AccessDef) parameters.getResultObject(BdfInstructionConstants.ACCESSDEF_OBJ);;
        if (accessDef == null) {
            accessDef = BdfInstructionUtils.getMandatoryAccessDef(parameters.getBdfServer().getAccessManager(), parameters.getRequestMap());
        }
        return accessDef;
    }

    public static TableExportDescription getTableExportDescription(OutputParameters parameters) throws ErrorMessageException {
        TableExportDescription tableExportDescription = (TableExportDescription) parameters.getResultObject(BdfInstructionConstants.TABLEEXPORTDESCRIPTION_OBJ);
        if (tableExportDescription == null) {
            tableExportDescription = BdfInstructionUtils.getMandatoryTableExportDescription(parameters.getBdfServer().getTableExportManager(), parameters.getRequestMap());
        }
        return tableExportDescription;
    }

    public static TransformationDescription getTransformationDescription(OutputParameters parameters) throws ErrorMessageException {
        TransformationDescription transformationDescription = (TransformationDescription) parameters.getResultObject(BdfInstructionConstants.TRANSFORMATIONDESCRIPTION_OBJ);
        if (transformationDescription == null) {
            transformationDescription = BdfInstructionUtils.getMandatoryTransformationDescription(parameters.getBdfServer(), parameters.getRequestMap());
        }
        return transformationDescription;
    }

    public static TemplateDescription getTemplateDescription(OutputParameters parameters) throws ErrorMessageException {
        TemplateDescription templateDescription = (TemplateDescription) parameters.getResultObject(BdfInstructionConstants.TEMPLATEDESCRIPTION_OBJ);
        if (templateDescription == null) {
            templateDescription = BdfInstructionUtils.getMandatoryTemplateDescription(parameters.getBdfServer(), parameters.getRequestMap());
        }
        return templateDescription;
    }

    public static PathConfiguration getPathConfiguration(OutputParameters parameters) {
        PathConfiguration pathConfiguration = (PathConfiguration) parameters.getResultObject(BdfInstructionConstants.PATHCONFIGURATION_OBJ);
        if (pathConfiguration == null) {
            pathConfiguration = PathConfigurationBuilder.build(parameters.getBdfServer());
        }
        return pathConfiguration;
    }

    public static BalayageDescription getBalayageDescription(OutputParameters parameters) throws ErrorMessageException {
        BalayageDescription balayageDescription = (BalayageDescription) parameters.getResultObject(BdfInstructionConstants.BALAYAGEDESCRIPTION_OBJ);
        if (balayageDescription == null) {
            balayageDescription = BdfInstructionUtils.getMandatoryBalayageDescription(parameters.getBdfServer().getBalayageManager(), parameters.getRequestMap());
        }
        return balayageDescription;
    }

    public static TemplateContentDescription getTemplateContentDescription(OutputParameters parameters) throws ErrorMessageException {
        TemplateContentDescription templateContentDescription = (TemplateContentDescription) parameters.getResultObject(BdfInstructionConstants.TEMPLATECONTENTDESCRIPTION_OBJ);
        if (templateContentDescription == null) {
            TemplateDescription templateDescription = (TemplateDescription) parameters.getResultObject(BdfInstructionConstants.TEMPLATEDESCRIPTION_OBJ);
            if (templateDescription == null) {
                templateDescription = BdfInstructionUtils.getMandatoryTemplateDescription(parameters.getBdfServer(), parameters.getRequestMap());
            }
            templateContentDescription = BdfInstructionUtils.getMandatoryTemplateContentDescription(templateDescription, parameters.getRequestMap());
        }
        return templateContentDescription;
    }

    public static Balayage getBalayage(OutputParameters parameters) throws ErrorMessageException {
        Balayage balayage = (Balayage) parameters.getResultObject(BdfInstructionConstants.BALAYAGE_OBJ);
        if (balayage == null) {
            balayage = BdfInstructionUtils.getMandatoryBalayage(parameters.getBdfServer().getBalayageManager(), parameters.getRequestMap());
        }
        return balayage;
    }

    public static RelativePath getRelativePath(OutputParameters parameters) throws ErrorMessageException {
        RelativePath path = (RelativePath) parameters.getResultObject(BdfInstructionConstants.RELATIVEPATH_OBJ);
        if (path == null) {
            path = BdfInstructionUtils.getMandatoryResourcePath(parameters.getRequestMap());
        }
        return path;
    }


}
